import React, { useEffect, useState } from 'react';

import { PageContainer } from '@ant-design/pro-layout';

import {
  Card,
  Table,
  Row,
  Col,
  Button,
  Space,
  message,
  Image,
  Tag,
  Divider,
  Popconfirm,
} from 'antd';

import WeappModal from './modal';

import { addWeappList, getWeappList, deleteWeappList } from '@/services/weapp';

import { Pagination } from '@/types/common';

import { WeappItem } from '@/types/index';

interface IState {
  visable: boolean;
  type: 'add' | 'edit';
  queryParams: Pagination;
  weappList: WeappItem[];
  curWeapp: WeappItem;
  loading: boolean;
}

export interface WeappInfo {
  appId: string;
  cover: string;
  name: string;
}

const Weapp: React.FC = () => {
  const [state, setState] = useState<IState>({
    visable: false,
    type: 'add',
    queryParams: {
      isPage: 1,
      page: 1,
      pageSize: 10,
      total: 0,
    },
    loading: false,
    weappList: [],
    curWeapp: { appId: '', cover: '', name: '', id: '' },
  });

  const { visable, type, queryParams, weappList, curWeapp, loading } = state;
  console.log(
    visable,
    type,
    queryParams,
    weappList,
    curWeapp,
    ' visable, type, queryParams, weappList, curWeapp',
  );

  useEffect(() => {
    GetWeappList();
  }, [queryParams.page]);

  // 打开弹窗
  const openModal = () => {
    setState({
      ...state,
      visable: true,
      type: 'add',
    });
  };

  // 关闭弹窗
  const closeModal = () => {
    setState({
      ...state,
      curWeapp: { appId: '', cover: '', name: '', id: '' },
      visable: false,
    });
  };
  const onSubmit = (_records: WeappInfo) => {
    AddWeappList(_records);
  };

  // 翻页
  const changePagintion = (page: number) => {
    setState({
      ...state,
      queryParams: {
        page: page,
        pageSize: 10,
        isPage: 1,
        total: queryParams.total,
      },
    });
  };

  // 编辑小程序信息
  const editWeapp = (_records: WeappItem) => {
    console.log(_records);
    setState({
      ...state,
      visable: true,
      type: 'edit',
      curWeapp: _records,
    });
  };

  // 查询小程序
  const GetWeappList = async () => {
    setState({
      ...state,
      loading: true,
    });
    const { data, status } = await getWeappList(queryParams);
    if (status === 200) {
      console.log(data);
      setState({
        ...state,
        curWeapp: { appId: '', cover: '', name: '', id: '' },
        weappList: data.entityList,
        visable: false,
        loading: false,
        queryParams: {
          pageSize: 10,
          page: queryParams.page,
          total: data.total,
          isPage: 1,
        },
      });
    }
  };

  // 增加小程序
  const AddWeappList = async (_records: WeappInfo) => {
    const $par = {
      ..._records,
      type,
    };
    const { status } = await addWeappList($par);
    if (status === 200) {
      message.success(type === 'add' ? '添加成功' : '编辑成功');
      GetWeappList();
    }
  };
  // 删除小程序
  const DeleteWeappList = async (id: string) => {
    const { status } = await deleteWeappList(id);
    if (status === 200) {
      message.success('删除成功');
      GetWeappList();
    }
  };

  const columns = [
    {
      title: '小程序图标',
      key: 'cover',
      dataIndex: 'cover',
      render: (cover: string) => <Image width={60} height={60} src={cover} />,
    },
    {
      title: '小程序名字',
      key: 'name',
      dataIndex: 'name',
      render: (name: string) => <Tag color="cyan">{name}</Tag>,
    },
    {
      title: 'appId',
      key: 'appId',
      dataIndex: 'appId',
    },
    {
      title: '操作',
      dataIndex: 'action',
      key: 'action',
      render: (text, records) => (
        <Space>
          <a onClick={() => editWeapp(records)}>编辑</a>
          <Divider type="vertical" />
          <Popconfirm
            title="确定要删除这个小程序吗？"
            onConfirm={() => DeleteWeappList(records.id)}
          >
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];

  return (
    <PageContainer title="小程序管理">
      <Card>
        <Row style={{ marginBottom: '20px' }}>
          <Col push={22}>
            <Button type="primary" onClick={openModal}>
              新增小程序
            </Button>
          </Col>
        </Row>
        <Table
          loading={loading}
          dataSource={weappList}
          columns={columns}
          pagination={{
            total: queryParams.total,
            defaultCurrent: 1,
            current: queryParams.page,
            onChange: changePagintion,
          }}
        />
      </Card>

      <WeappModal
        visible={visable}
        onSubmit={onSubmit}
        closeModal={closeModal}
        curWeapp={curWeapp}
        type={type}
      />
    </PageContainer>
  );
};

export default Weapp;
